@@ -1,37 +1,33 @@
 /**
- * SelectionOverrides.js
- *
- * Released under LGPL License.
- * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
- *
- * License: http://www.tinymce.com/license
- * Contributing: http://www.tinymce.com/contributing
+ * Copyright (c) Tiny Technologies, Inc. All rights reserved.
+ * Licensed under the LGPL or a commercial license.
+ * For LGPL see License.txt in the project root for license information.
+ * For commercial licenses see https://www.tiny.cloud/
  */
 
-import { Arr } from '@ephox/katamari';
-import { Remove, Element as SugarElement, Attr, SelectorFilter, SelectorFind } from '@ephox/sugar';
-import DragDropOverrides from './DragDropOverrides';
-import EditorView from './EditorView';
+import { Element, HTMLElement, MouseEvent, Node, Range } from '@ephox/dom-globals';
+import { Arr, Obj } from '@ephox/katamari';
+import { Attr, Element as SugarElement, Remove, SelectorFilter, SelectorFind } from '@ephox/sugar';
+import { Editor } from 'tinymce/core/api/Editor';
+import EditorFocus from 'tinymce/core/focus/EditorFocus';
 import Env from './api/Env';
+import VK from './api/util/VK';
 import * as CaretContainer from './caret/CaretContainer';
 import CaretPosition from './caret/CaretPosition';
+import { isAfterContentEditableFalse, isBeforeContentEditableFalse } from './caret/CaretPositionPredicates';
 import * as CaretUtils from './caret/CaretUtils';
 import { CaretWalker } from './caret/CaretWalker';
+import { FakeCaret, isFakeCaretTarget } from './caret/FakeCaret';
 import * as LineUtils from './caret/LineUtils';
 import NodeType from './dom/NodeType';
 import RangePoint from './dom/RangePoint';
+import DragDropOverrides from './DragDropOverrides';
+import EditorView from './EditorView';
 import CefFocus from './focus/CefFocus';
 import * as CefUtils from './keyboard/CefUtils';
-import VK from './api/util/VK';
-import { FakeCaret, isFakeCaretTarget } from './caret/FakeCaret';
-import { Editor } from 'tinymce/core/api/Editor';
-import EditorFocus from 'tinymce/core/focus/EditorFocus';
-import { Range, Element, Node, HTMLElement, MouseEvent } from '@ephox/dom-globals';
 
 const isContentEditableTrue = NodeType.isContentEditableTrue;
 const isContentEditableFalse = NodeType.isContentEditableFalse;
-const isAfterContentEditableFalse = CaretUtils.isAfterContentEditableFalse;
-const isBeforeContentEditableFalse = CaretUtils.isBeforeContentEditableFalse;
 
 interface SelectionOverrides {
   showCaret: (direction: number, node: Element, before: boolean, scrollIntoView?: boolean) => Range;
@@ -292,9 +288,11 @@ const SelectionOverrides = function (editor: Editor): SelectionOverrides {
     });
 
     editor.on('setSelectionRange', function (e) {
-      let rng;
+      // If the range is set inside a short ended element, then move it
+      // to the side as IE for example will try to add content inside
+      e.range = normalizeShortEndedElementSelection(e.range);
 
-      rng = setContentEditableSelection(e.range, e.forward);
+      const rng = setContentEditableSelection(e.range, e.forward);
       if (rng) {
         e.range = rng;
       }
@@ -367,6 +365,37 @@ const SelectionOverrides = function (editor: Editor): SelectionOverrides {
     return isWithinCaretContainer(rng.startContainer) || isWithinCaretContainer(rng.endContainer);
   };
 
+  const normalizeShortEndedElementSelection = (rng: Range) => {
+    const shortEndedElements = editor.schema.getShortEndedElements();
+    const newRng = editor.dom.createRng();
+    const startContainer = rng.startContainer;
+    const startOffset = rng.startOffset;
+    const endContainer = rng.endContainer;
+    const endOffset = rng.endOffset;
+
+    if (Obj.has(shortEndedElements, startContainer.nodeName.toLowerCase())) {
+      if (startOffset === 0) {
+        newRng.setStartBefore(startContainer);
+      } else {
+        newRng.setStartAfter(startContainer);
+      }
+    } else {
+      newRng.setStart(startContainer, startOffset);
+    }
+
+    if (Obj.has(shortEndedElements, endContainer.nodeName.toLowerCase())) {
+      if (endOffset === 0) {
+        newRng.setEndBefore(endContainer);
+      } else {
+        newRng.setEndAfter(endContainer);
+      }
+    } else {
+      newRng.setEnd(endContainer, endOffset);
+    }
+
+    return newRng;
+  };
+
   const setContentEditableSelection = function (range: Range, forward?: boolean) {
     let node;
     const $ = editor.$;
@@ -421,7 +450,7 @@ const SelectionOverrides = function (editor: Editor): SelectionOverrides {
       return null;
     }
 
-    if (endOffset === startOffset + 1) {
+    if (endOffset === startOffset + 1 && startContainer === range.endContainer) {
       node = startContainer.childNodes[startOffset];
     }
 
