@@ -1,11 +1,8 @@
 /**
- * EditorManager.js
- *
- * Released under LGPL License.
- * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
- *
- * License: http://www.tinymce.com/license
- * Contributing: http://www.tinymce.com/contributing
+ * Copyright (c) Tiny Technologies, Inc. All rights reserved.
+ * Licensed under the LGPL or a commercial license.
+ * For LGPL see License.txt in the project root for license information.
+ * For commercial licenses see https://www.tiny.cloud/
  */
 
 import { Arr, Type } from '@ephox/katamari';
@@ -49,11 +46,15 @@ const isValidLegacyKey = function (id) {
 };
 
 const globalEventDelegate = function (e) {
+  const type = e.type;
   each(EditorManager.get(), function (editor) {
-    if (e.type === 'scroll') {
-      editor.fire('ScrollWindow', e);
-    } else {
-      editor.fire('ResizeWindow', e);
+    switch (type) {
+      case 'scroll':
+        editor.fire('ScrollWindow', e);
+        break;
+      case 'resize':
+        editor.fire('ResizeWindow', e);
+        break;
     }
   });
 };
@@ -177,7 +178,7 @@ EditorManager = {
 
   setup () {
     const self = this;
-    let baseURL, documentBaseURL, suffix = '', preInit, src;
+    let baseURL, documentBaseURL, suffix = '';
 
     // Get base URL for the current document
     documentBaseURL = URI.getDocumentBaseUrl(document.location);
@@ -193,7 +194,7 @@ EditorManager = {
     }
 
     // If tinymce is defined and has a base use that or use the old tinyMCEPreInit
-    preInit = window.tinymce || window.tinyMCEPreInit;
+    const preInit = window.tinymce || window.tinyMCEPreInit;
     if (preInit) {
       baseURL = preInit.base || preInit.baseURL;
       suffix = preInit.suffix;
@@ -201,7 +202,10 @@ EditorManager = {
       // Get base where the tinymce script is located
       const scripts = document.getElementsByTagName('script');
       for (let i = 0; i < scripts.length; i++) {
-        src = scripts[i].src;
+        const src = scripts[i].src || '';
+        if (src === '') {
+          continue;
+        }
 
         // Script types supported:
         // tinymce.js tinymce.min.js tinymce.dev.js
@@ -221,7 +225,7 @@ EditorManager = {
       // We didn't find any baseURL by looking at the script elements
       // Try to use the document.currentScript as a fallback
       if (!baseURL && document.currentScript) {
-        src = (<any> document.currentScript).src;
+        const src = (<any> document.currentScript).src;
 
         if (src.indexOf('.min') !== -1) {
           suffix = '.min';
