@@ -1,11 +1,8 @@
 /**
- * InsertBlock.js
- *
- * Released under LGPL License.
- * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
- *
- * License: http://www.tinymce.com/license
- * Contributing: http://www.tinymce.com/contributing
+ * Copyright (c) Tiny Technologies, Inc. All rights reserved.
+ * Licensed under the LGPL or a commercial license.
+ * For LGPL see License.txt in the project root for license information.
+ * For commercial licenses see https://www.tiny.cloud/
  */
 
 import Settings from '../api/Settings';
@@ -16,11 +13,24 @@ import InsertLi from './InsertLi';
 import NewLineUtils from './NewLineUtils';
 import NormalizeRange from '../selection/NormalizeRange';
 import Zwsp from '../text/Zwsp';
-import Tools from '../api/util/Tools';
 import { isCaretNode } from 'tinymce/core/fmt/FormatContainer';
+import DOMUtils from '../api/dom/DOMUtils';
+import { Element as DomElement, DocumentFragment, KeyboardEvent } from '@ephox/dom-globals';
+import { PredicateFilter, Element, Node, Css } from '@ephox/sugar';
+import { Arr, Option, Options, Obj } from '@ephox/katamari';
+import { Editor } from '../api/Editor';
+import { EditorEvent } from '../api/dom/EventUtils';
+import Bookmarks from '../bookmark/Bookmarks';
 
-const isEmptyAnchor = function (elm) {
-  return elm && elm.nodeName === 'A' && Tools.trim(Zwsp.trim(elm.innerText || elm.textContent)).length === 0;
+const trimZwsp = (fragment: DocumentFragment) => {
+  Arr.each(PredicateFilter.descendants(Element.fromDom(fragment), Node.isText), (text) => {
+    const rawNode = text.dom();
+    rawNode.nodeValue = Zwsp.trim(rawNode.nodeValue);
+  });
+};
+
+const isEmptyAnchor = function (dom: DOMUtils, elm: DomElement) {
+  return elm && elm.nodeName === 'A' && dom.isEmpty(elm);
 };
 
 const isTableCell = function (node) {
@@ -71,7 +81,7 @@ const trimInlineElementsOnLeftSideOfBlock = function (dom, nonEmptyElementsMap,
     if (!node.hasChildNodes() || (node.firstChild === node.lastChild && node.firstChild.nodeValue === '')) {
       dom.remove(node);
     } else {
-      if (isEmptyAnchor(node)) {
+      if (isEmptyAnchor(dom, node)) {
         dom.remove(node);
       }
     }
@@ -123,11 +133,37 @@ const getEditableRoot = function (dom, node) {
   return parent !== root ? editableRoot : root;
 };
 
-const setForcedBlockAttrs = function (editor, node) {
+const applyAttributes = (editor: Editor, node: DomElement, forcedRootBlockAttrs: Record<string, string>) => {
+  // Merge and apply style attribute
+  Option.from(forcedRootBlockAttrs.style)
+    .map(editor.dom.parseStyle)
+    .each((attrStyles) => {
+      const currentStyles = Css.getAllRaw(Element.fromDom(node));
+      const newStyles = { ...currentStyles, ...attrStyles };
+      editor.dom.setStyles(node, newStyles);
+    });
+
+  // Merge and apply class attribute
+  const attrClassesOpt = Option.from(forcedRootBlockAttrs.class).map((attrClasses) => attrClasses.split(/\s+/));
+  const currentClassesOpt = Option.from(node.className).map((currentClasses) => Arr.filter(currentClasses.split(/\s+/), (clazz) => clazz !== ''));
+  Options.lift2(attrClassesOpt, currentClassesOpt, (attrClasses, currentClasses) => {
+    const filteredClasses = Arr.filter(currentClasses, (clazz) => !Arr.contains(attrClasses, clazz));
+    const newClasses = [...attrClasses, ...filteredClasses];
+    editor.dom.setAttrib(node, 'class', newClasses.join(' '));
+  });
+
+  // Apply any remaining forced root block attributes
+  const appliedAttrs = ['style', 'class'];
+  const remainingAttrs = Obj.bifilter(forcedRootBlockAttrs, (_, attrs) => !Arr.contains(appliedAttrs, attrs)).t;
+  editor.dom.setAttribs(node, remainingAttrs);
+};
+
+const setForcedBlockAttrs = function (editor: Editor, node) {
   const forcedRootBlockName = Settings.getForcedRootBlock(editor);
 
   if (forcedRootBlockName && forcedRootBlockName.toLowerCase() === node.tagName.toLowerCase()) {
-    editor.dom.setAttribs(node, Settings.getForcedRootBlockAttrs(editor));
+    const forcedRootBlockAttrs = Settings.getForcedRootBlockAttrs(editor);
+    applyAttributes(editor, node, forcedRootBlockAttrs);
   }
 };
 
@@ -206,7 +242,7 @@ const addBrToBlockIfNeeded = function (dom, block) {
   }
 };
 
-const insert = function (editor, evt) {
+const insert = function (editor: Editor, evt: EditorEvent<KeyboardEvent>) {
   let tmpRng, editableRoot, container, offset, parentBlock, shiftKey;
   let newBlock, fragment, containerBlock, parentBlockName, containerBlockName, newBlockName, isAfterLastNodeInContainer;
   const dom = editor.dom;
@@ -221,7 +257,6 @@ const insert = function (editor, evt) {
 
     if (name || parentBlockName === 'TABLE' || parentBlockName === 'HR') {
       block = dom.create(name || newBlockName);
-      setForcedBlockAttrs(editor, block);
     } else {
       block = parentBlock.cloneNode(false);
     }
@@ -235,7 +270,8 @@ const insert = function (editor, evt) {
       // Clone any parent styles
       do {
         if (textInlineElements[node.nodeName]) {
-          if (isCaretNode(node)) {
+          // Ignore caret or bookmark nodes when cloning
+          if (isCaretNode(node) || Bookmarks.isBookmarkNode(node)) {
             continue;
           }
 
@@ -253,6 +289,8 @@ const insert = function (editor, evt) {
       } while ((node = node.parentNode) && node !== editableRoot);
     }
 
+    setForcedBlockAttrs(editor, block);
+
     emptyBlock(caretNode);
 
     return block;
@@ -415,6 +453,7 @@ const insert = function (editor, evt) {
     if (dom.isEmpty(parentBlock)) {
       emptyBlock(parentBlock);
     }
+    setForcedBlockAttrs(editor, newBlock);
     NewLineUtils.moveToCaretPosition(editor, newBlock);
   } else if (isCaretAtStartOrEndOfBlock()) {
     insertNewBlockAfter();
@@ -427,6 +466,7 @@ const insert = function (editor, evt) {
     tmpRng = includeZwspInRange(rng).cloneRange();
     tmpRng.setEndAfter(parentBlock);
     fragment = tmpRng.extractContents();
+    trimZwsp(fragment);
     trimLeadingLineBreaks(fragment);
     newBlock = fragment.firstChild;
     dom.insertAfter(fragment, parentBlock);
@@ -444,6 +484,7 @@ const insert = function (editor, evt) {
       dom.remove(newBlock);
       insertNewBlockAfter();
     } else {
+      setForcedBlockAttrs(editor, newBlock);
       NewLineUtils.moveToCaretPosition(editor, newBlock);
     }
   }
