@@ -1,24 +1,21 @@
 /**
- * VisualChars.js
- *
- * Released under LGPL License.
- * Copyright (c) 1999-2017 Ephox Corp. All rights reserved
- *
- * License: http://www.tinymce.com/license
- * Contributing: http://www.tinymce.com/contributing
+ * Copyright (c) Tiny Technologies, Inc. All rights reserved.
+ * Licensed under the LGPL or a commercial license.
+ * For LGPL see License.txt in the project root for license information.
+ * For commercial licenses see https://www.tiny.cloud/
  */
 
-import Data from './Data';
-import Nodes from './Nodes';
 import { Arr } from '@ephox/katamari';
 import { Element, Node } from '@ephox/sugar';
+import Data from './Data';
+import Nodes from './Nodes';
 
 const show = function (editor, rootElm) {
   let node, div;
   const nodeList = Nodes.filterDescendants(Element.fromDom(rootElm), Nodes.isMatch);
 
   Arr.each(nodeList, function (n) {
-    const withSpans = Nodes.replaceWithSpans(Node.value(n));
+    const withSpans = Nodes.replaceWithSpans(editor.dom.encode(Node.value(n)));
 
     div = editor.dom.create('div', null, withSpans);
     while ((node = div.lastChild)) {
